{
  Solutions to the Advent Of Code.
  Copyright (C) 2023  Stefan Müller

  This program is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  this program.  If not, see <http://www.gnu.org/licenses/>.
}

unit UStepCounterTestCases;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testregistry, USolver, UBaseTestCases, UStepCounter;

type

  { TStepCounterFullDataTestCase }

  TStepCounterFullDataTestCase = class(TEngineBaseTest)
  protected
    function CreateSolver: ISolver; override;
  published
    procedure TestPart1;
  end;

  { TStepCounterMaxStepsExampleTestCase }

  TStepCounterMaxStepsExampleTestCase = class(TExampleEngineBaseTest)
  protected
    function CreateSolver: ISolver; override;
    function GetMaxSteps: Integer; virtual; abstract;
  end;

  { TStepCounterMax6ExampleTestCase }

  TStepCounterMax6ExampleTestCase = class(TStepCounterMaxStepsExampleTestCase)
  protected
    function GetMaxSteps: Integer; override;
  published
    procedure TestPart1;
    procedure TestPart2;
  end;

  { TStepCounterMax10ExampleTestCase }

  TStepCounterMax10ExampleTestCase = class(TStepCounterMaxStepsExampleTestCase)
  protected
    function GetMaxSteps: Integer; override;
  published
    procedure TestPart2;
  end;

  { TStepCounterMax50ExampleTestCase }

  TStepCounterMax50ExampleTestCase = class(TStepCounterMaxStepsExampleTestCase)
  protected
    function GetMaxSteps: Integer; override;
  published
    procedure TestPart2;
  end;

  { TStepCounterMax100ExampleTestCase }

  TStepCounterMax100ExampleTestCase = class(TStepCounterMaxStepsExampleTestCase)
  protected
    function GetMaxSteps: Integer; override;
  published
    procedure TestPart2;
  end;

  { TStepCounterMax500ExampleTestCase }

  TStepCounterMax500ExampleTestCase = class(TStepCounterMaxStepsExampleTestCase)
  protected
    function GetMaxSteps: Integer; override;
  published
    procedure TestPart2;
  end;

  { TStepCounterMax1000ExampleTestCase }

  TStepCounterMax1000ExampleTestCase = class(TStepCounterMaxStepsExampleTestCase)
  protected
    function GetMaxSteps: Integer; override;
  published
    procedure TestPart2;
  end;

  { TStepCounterMax5000ExampleTestCase }

  TStepCounterMax5000ExampleTestCase = class(TStepCounterMaxStepsExampleTestCase)
  protected
    function GetMaxSteps: Integer; override;
  published
    procedure TestPart2;
  end;

implementation

{ TStepCounterFullDataTestCase }

function TStepCounterFullDataTestCase.CreateSolver: ISolver;
begin
  Result := TStepCounter.Create;
end;

procedure TStepCounterFullDataTestCase.TestPart1;
begin
  AssertEquals(3809, FSolver.GetResultPart1);
end;

{ TStepCounterMaxStepsExampleTestCase }

function TStepCounterMaxStepsExampleTestCase.CreateSolver: ISolver;
var
  solver: TStepCounter;
begin
  solver := TStepCounter.Create;
  solver.MaxSteps := GetMaxSteps;
  Result := solver;
end;

{ TStepCounterMax6ExampleTestCase }

function TStepCounterMax6ExampleTestCase.GetMaxSteps: Integer;
begin
  Result := 6;
end;

procedure TStepCounterMax6ExampleTestCase.TestPart1;
begin
  AssertEquals(16, FSolver.GetResultPart1);
end;

procedure TStepCounterMax6ExampleTestCase.TestPart2;
begin
  AssertEquals(16, FSolver.GetResultPart2);
end;

{ TStepCounterMax10ExampleTestCase }

function TStepCounterMax10ExampleTestCase.GetMaxSteps: Integer;
begin
  Result := 10;
end;

procedure TStepCounterMax10ExampleTestCase.TestPart2;
begin
  AssertEquals(50, FSolver.GetResultPart2);
end;

{ TStepCounterMax50ExampleTestCase }

function TStepCounterMax50ExampleTestCase.GetMaxSteps: Integer;
begin
  Result := 50;
end;

procedure TStepCounterMax50ExampleTestCase.TestPart2;
begin
  AssertEquals(1594, FSolver.GetResultPart2);
end;

{ TStepCounterMax100ExampleTestCase }

function TStepCounterMax100ExampleTestCase.GetMaxSteps: Integer;
begin
  Result := 100;
end;

procedure TStepCounterMax100ExampleTestCase.TestPart2;
begin
  AssertEquals(6536, FSolver.GetResultPart2);
end;

{ TStepCounterMax500ExampleTestCase }

function TStepCounterMax500ExampleTestCase.GetMaxSteps: Integer;
begin
  Result := 500;
end;

procedure TStepCounterMax500ExampleTestCase.TestPart2;
begin
  AssertEquals(167004, FSolver.GetResultPart2);
end;

{ TStepCounterMax1000ExampleTestCase }

function TStepCounterMax1000ExampleTestCase.GetMaxSteps: Integer;
begin
  Result := 1000;
end;

procedure TStepCounterMax1000ExampleTestCase.TestPart2;
begin
  AssertEquals(668697, FSolver.GetResultPart2);
end;

{ TStepCounterMax5000ExampleTestCase }

function TStepCounterMax5000ExampleTestCase.GetMaxSteps: Integer;
begin
  Result := 5000;
end;

procedure TStepCounterMax5000ExampleTestCase.TestPart2;
begin
  AssertEquals(16733044, FSolver.GetResultPart2);
end;

initialization

  RegisterTest(TStepCounterFullDataTestCase);
  RegisterTest(TStepCounterMax6ExampleTestCase);
  RegisterTest(TStepCounterMax10ExampleTestCase);
  RegisterTest(TStepCounterMax50ExampleTestCase);
  RegisterTest(TStepCounterMax100ExampleTestCase);
  RegisterTest(TStepCounterMax500ExampleTestCase);
  RegisterTest(TStepCounterMax1000ExampleTestCase);
  RegisterTest(TStepCounterMax5000ExampleTestCase);
end.

